#!/bin/sh
# SPDX-License-Identifier: MIT

usage() {
    echo "
USAGE:
  $(basename "$0") [OPTIONS] [OUTPUT]

  OPTIONS:
    -i|--input <file>               -- name of the file for which
                                        the sbom is to be created
    -c|--config <yaml>              -- file in yaml format for creating sbom.
                                        default value is config.yaml
    -d|--debug                      -- set log output level to debug
    -e|--external_sbom <spdx.json>  -- external .sbom.json to add to
                                        the sbom you are creating
    -o|--output <sbom file name>    -- file name of the sbom to be created
    -p|--package <package_list>     -- *package_list.txt created by build_rootfs
                                        e.g. baseos-x2-3.18.4-at.5.package_list.txt
    -h|--help                       -- this output

  OUTPUT:
    created sbom.json (<input>.sbom.json)
"
}

error() {
    echo "$@" >&2
    exit 1
}

required_input() {
    usage >&2
    error "-i|--input <file> option is a required parameter"
}

SCRIPTDIR="$(realpath -e "$(dirname "$0")")"

input=""
output=""
config="$SCRIPTDIR/config.yaml"
debug=""
external_sbom=""
package=""

# parse args
while [ $# -ne 0 ]; do
    case "$1" in
    "-i" | "--input")
        [ $# -lt 2 ] && error "$1 requires an argument"
        input=$(realpath -e "$2") ||
            error "input $2 does not exist"
        shift
        ;;
    "-c" | "--config")
        [ $# -lt 2 ] && error "$1 requires an argument"
        config="$2"
        shift
        ;;
    "-d" | "--debug")
        debug=1
        ;;
    "-e" | "--external_sbom")
        [ $# -lt 2 ] && error "$1 requires an argument"
        external_sbom=$(realpath -e "$2") ||
            error "external-sbom $2 does not exist"
        shift
        ;;
    "-o" | "--output")
        [ $# -lt 2 ] && error "$1 requires an argument"
        output="$2"
        shift
        ;;
    "-p" | "--package")
        [ $# -lt 2 ] && error "$1 requires an argument"
        package=$(realpath -e "$2") ||
            error "package $2 does not exist"
        shift
        ;;
    "-h" | "--help")
        usage
        exit 0
        ;;
    *)
        echo "Invalid option $1" >&2
        usage >&2
        exit 1
        ;;
    esac
    shift
done

[ -z "$input" ] && required_input
# If output is not specified, SBOM is output to the current directory
: "${output:=$PWD/$(basename "$input")}"

deps="$SCRIPTDIR/deps"
# shellcheck disable=SC3013 # dash, busybox ash and mksh all support -nt
if ! [ -e "$deps/.stamp" ] ||
    [ "$SCRIPTDIR/requirements.txt" -nt "$deps/.stamp" ] ||
    ! PYTHONPATH="$deps" python3 -m spdx_tools.spdx.clitools.pyspdxtools --help >/dev/null 2>&1; then
    # install if missing, or update if requirements changed,
    # or could not use it
    rm -rf "$deps"
    python3 -m pip install --target "$deps" -r "$SCRIPTDIR/requirements.txt" ||
        error "Could not install python deps - missing python or pip?"
    touch "$deps/.stamp"
fi

# make_sbom.py currently only works with input in current directory
cd "$(dirname "$input")" || exit
PYTHONPATH="$deps" \
    python3 "$SCRIPTDIR/make_sbom.py" \
    -i "$(basename "$input")" -c "$config" -o "$output" \
    ${debug:+-d} ${external_sbom:+-e "$external_sbom"} \
    ${package:+-p "$package"}
